//fbx.initialize
struct GSin
{
    //fbx.vertex.structbody
};

GSin VS(uint vid : SV_VertexID, uint iid : SV_InstanceID)
{
    GSin o;

	//fbx.vertex.fill

    #if defined(HAS_INSTANCEID)
    o.instanceid = iid;
    #endif
    uint ssid = o.subsetid;
	sShapeMeta smeta = ShapeMeta[ssid];
    float3 opos = o.position;
    float3 onorm = o.normal;
    #if defined(HAS_TANGENT)
    float3 otang = o.tangent.xyz;
    float3 obinorm = o.binormal.xyz;
    #endif
	#if defined(HAS_PREVPOS)
	o.prevpos = o.position;
	#endif

    #if defined(DOBLENDSHAPES)
    if(smeta.ShapeCount != 0)
    {
        uint spvmi = o.subsetvertexid * SHAPEVERTSIZE + smeta.FirstShapeOffset;
    	for(uint i=0; i<smeta.ShapeCount; i++)
    	{
    		uint sva = spvmi + i * smeta.ShapeSize;
    		uint wa = i * 8 + iid * smeta.ShapeCount * 8;

    		float weight = asfloat(ShapeWeights.Load(wa));
            if(weight > 0.0001)
            {
        		//float weight = tempw;
        		float3 posdelta = asfloat(ShapeData.Load3(sva + 0)) - opos;
        		o.position += posdelta * weight;

        		#if defined(SHAPENORMALS)
        		float3 normdelta = asfloat(ShapeData.Load3(sva + 12)) - onorm;
        		o.normal += normdelta * weight;
        		#endif

        		#if defined(HAS_PREVPOS)
        		float pweight = asfloat(ShapeWeights.Load(wa + 4));
        		o.prevpos += posdelta * pweight;
        		#endif
            }
        }
        o.normal = normalize(o.normal);
    }
    #endif

    #if defined(DOSKINNING)
    uint2 winfo = SkinningWeights.Load2(vid * VERTWEIGHTSIZE);
    if(winfo.x != 0)
    {
        float3 npos = 0;
        float3 nnorm = 0;
        #if defined(HAS_TANGENT)
    	bool hastans = (o.featureflags & FLAG_TANGENT) > 0;
        float3 ntang = 0;
        float3 nbinorm = 0;
        #endif
        #if defined(HAS_PREVPOS)
        float3 nppos = 0;
        #endif
        float weightsum = 1;

        for(uint i=0; i<winfo.x; i++)
        {
            uint2 wpair = SkinningWeights.Load2(vid * VERTWEIGHTSIZE + 8 + i*8);
            float weight = asfloat(wpair.y);
            //weightsum = max(weightsum, weight);
            uint bid = wpair.x;
	        BoneBinding binding = BoneBindingInfo[bid];

            float4x4 invmat = binding.InverseBindingMatrix;
        	float4x4 bonemat = mul(invmat, BoneMatrices[binding.BoneID + iid * BONESETSIZE]);

        	npos += mul(float4(o.position, 1), bonemat).xyz * weight;
        	nnorm += mul(float4(o.normal, 0), bonemat).xyz * weight;

            #if defined(HAS_TANGENT)
        	if(hastans)
        	{
        		ntang += mul(float4(o.tangent.xyz, 0), bonemat).xyz * weight;
        		nbinorm += mul(float4(o.binormal.xyz, 0), bonemat).xyz * weight;
            }
            #endif
            #if defined(HAS_PREVPOS)
        	float4x4 pbonemat = mul(invmat, PrevBoneMatrices[binding.BoneID + iid * BONESETSIZE]);
        	nppos += mul(float4(o.prevpos, 1), pbonemat).xyz * weight;
            #endif
        }
        o.position = npos/weightsum;
        o.normal = (all(nnorm)) ? normalize(nnorm/weightsum) : o.normal;
        #if defined(HAS_TANGENT)
        o.tangent.xyz = (all(ntang)) ? normalize(ntang/weightsum) : o.tangent.xyz;
        o.binormal.xyz = (all(nbinorm)) ? normalize(nbinorm/weightsum) : o.binormal.xyz;
        #endif
        #if defined(HAS_PREVPOS)
        o.prevpos = nppos/weightsum;
        #endif
    }
    #endif

    return o;
}
[maxvertexcount(3)]
void GS(triangle GSin input[3], inout TriangleStream<GSin>GSOut)
{
	GSin v = (GSin)0;

	for(uint i=0;i<3;i++)
	{
		v=input[i];
		GSOut.Append(v);
	}
}

GeometryShader StreamOutGS = ConstructGSWithSO( CompileShader( gs_5_0, GS() ), STREAMOUTLAYOUT );

technique11 Layout
{
	pass P0
	{

		SetVertexShader( CompileShader( vs_5_0, VS() ) );
		SetGeometryShader( CompileShader( gs_5_0, GS() ) );
	    SetGeometryShader( StreamOutGS );

	}
}
